// Modelo del horno no lineal
// 
// parámetros del sistema
R0=100;
a=3.9e-3;
Ka=20;
Ra=150;
Rb=150;
Kr=50; //Kr=100;
Rl=5;
Vcc=15;
Cp=10;
Ch=0.5; 
Rp=0.5; //Rp=1;
Rh=2;
Rf=50;
Lf=0.2; //Lf=0.5;
Rg=1;
Lg=0;
Kg=10; //Kg=5;

//Punto de equilibrio
// Ta=20ºC
// Vcc=15V
// Th=40ºC
// Se hacen cero las derivadas para los valores dados

// A continuación se define una función para encontrar elpunto de equilibrio. 
// Se reemplazan en el modelo las variabes que se conocen en el punto de equilibrio 
// en este caso Ta y Th.

function y=horno1(x)
    R0=100;
    a=3.9e-3;
    Ka=20;
    Ra=150;
    Rb=150;
    Kr=50; //Kr=100;
    Rl=5;
    Vcc=15;
    Cp=10;
    Ch=0.5; 
    Rp=0.5; //Rp=1;
    Rh=2;
    Rf=50;
    Lf=0.2; //Lf=0.5;
    Rg=1;
    Lg=0;
    Kg=10; //Kg=5;
    Ta=20;
    Th=40;
    didt=((Kr*x(3)/(Ra+Kr*x(3))-R0*(1+a*Th)/(Rb+R0*(1+a*Th)))*Vcc*Ka-x(1)*Rf)/Lf;  // dx(1)/dt
    dtldt=((x(1)*Kg/(Rg+Rl))^2*Rl-(x(2)-Th)/Rp)/Cp;                                // dx(2)/dt
    dthdt=((x(2)-Th)/Rp-(Th-Ta)/Rh)/Ch;                                            // dx(3)/dt
    y=[didt,dtldt,dthdt]
endfunction

// Se ejecuta fsolve para encontrar el punto de equilibrio.
[v,w]=fsolve([10,0,10],horno1)      // Los valores calculados son: If=0.848   Tl=45   Q=4.08

// función que representa el sistema de ecuaciones diferenciales no lineales.
// La variable Q es el ángulo del pote y Ta es la temperatura ambiente
//
clear
clc
xdel();
function y=horno(t,x)
    R0=100;
    a=3.9e-3;
    Ka=20;
    Ra=150;
    Rb=150;
    Kr=50; //Kr=100;
    Rl=5;
    Cp=10;
    Ch=0.5; 
    Rp=0.5; //Rp=1;
    Rh=2;
    Rf=50;
    Lf=0.2; //Lf=0.5;
    Rg=1;
    Lg=0;
    Kg=10; //Kg=5;
    Ta=20;
    Q=4.0865;  //Q=3.82
    Vcc=15;
    t0=00;
    didt=((Kr*Q*(t>t0)/(Ra+Kr*Q*(t>t0))-R0*(1+a*x(3))/(Rb+R0*(1+a*x(3))))*Vcc*Ka-x(1)*Rf)/Lf;
    dtldt=((x(1)*Kg/(Rg+Rl))^2*Rl-(x(2)-x(3))/Rp)/Cp;
    dthdt=((x(2)-x(3))/Rp-(x(3)-Ta)/Rh)/Ch;
    y=[didt;dtldt;dthdt];
endfunction
y0=[0;0;0];
t0=0;
tf=500;
t=linspace(t0,tf,5000);
// Lafunción ode resuelve el sistema de ecuaciones anterior.
ys=ode(y0,t0,t,horno);
subplot(311)
plot2d(t,ys(1,:))
ylabel("Corriente de Campo");
xgrid(4) 
 subplot(312)
plot2d(t,ys(2,:))
ylabel("Temp. de Calefactor");
xgrid(4)
subplot(313)
plot2d(t,ys(3,:))
ylabel("Temp. del Recinto");
xgrid(4)


// Cáculo del modelo lineal alrededor del punto de equilibrio
//
//
clear
clc
// Función con el modelo completo
function y=horno2(t,x)
    R0=100;
    a=3.9e-3;
    Ka=20;
    Ra=150;
    Rb=150;
    Kr=50; //Kr=100;
    Rl=5;
    Cp=10;
    Ch=0.5; 
    Rp=0.5; //Rp=1;
    Rh=2;
    Rf=50;
    Lf=0.2; //Lf=0.5;
    Rg=1;
    Lg=0;
    Kg=10; //Kg=5;
    Ta=20;
//    Q=3.82;
    Vcc=15;
//    t0=200;
    didt=((Kr*x(4)/(Ra+Kr*x(4))-R0*(1+a*x(3))/(Rb+R0*(1+a*x(3))))*Vcc*Ka-x(1)*Rf)/Lf;
    dtldt=((x(1)*Kg/(Rg+Rl))^2*Rl-(x(2)-x(3))/Rp)/Cp;
    dthdt=((x(2)-x(3))/Rp-(x(3)-x(5))/Rh)/Ch;
    y=[didt;dtldt;dthdt];
endfunction
X0=[0.8485281;45;40;4.0865414;20];
// numderivative calcula el jacobiano y el hessiano, se usa solamente el jacobiano
[J0,H0]=numderivative(horno2,X0);
A1=J0(:,1:3)
B1=J0(:,4:5)
P=poly(A1,"s");
V=roots(P)





// Se va a utilizar la función lin para sacar elmodelo lineal.
// La función se arma en función de X y U.
clear
clc
function [y,xdot]=horno3(x,u)
    R0=100;
    a=3.9e-3;
    Ka=20;
    Ra=150;
    Rb=150;
    Kr=50; //Kr=100;
    Rl=5;
    Cp=10;
    Ch=0.5; 
    Rp=0.5; //Rp=1;
    Rh=2;
    Rf=50;
    Lf=0.2; //Lf=0.5;
    Rg=1;
    Lg=0;
    Kg=10; //Kg=5;
    Ta=20;
//    Q=3.82;
    Vcc=15;
//    t0=200;
    xdot(1)=((Kr*u(1)/(Ra+Kr*u(1))-R0*(1+a*x(3))/(Rb+R0*(1+a*x(3))))*Vcc*Ka-x(1)*Rf)/Lf;
    xdot(2)=((x(1)*Kg/(Rg+Rl))^2*Rl-(x(2)-x(3))/Rp)/Cp;
    xdot(3)=((x(2)-x(3))/Rp-(x(3)-u(2))/Rh)/Ch;
    y=x(3);
endfunction
X0=[0.8485281;45;40];
U0=[4.0865414;20];
[A,B,C,D]=lin(horno3,X0,U0)
// Debería dar lo mismo que el caso anterior pero hay cierto error.
